@@ -27,12 +27,13 @@
 #include "cairo-support.h"
 
 static void
-murrine_draw_inset (cairo_t          *cr,
+murrine_draw_inset (cairo_t *cr,
                     const MurrineRGB *bg_color,
                     double x, double y, double w, double h,
                     double radius, uint8 corners)
 {
 	MurrineRGB highlight, shadow;
+	radius = MIN (radius, MIN (w/2.0, h/2.0));
 
 	murrine_shade (bg_color, 1.15, &highlight);
 	murrine_shade (bg_color, 0.4, &shadow);
@@ -82,7 +83,7 @@ murrine_draw_inset (cairo_t          *cr,
 
 static void
 murrine_draw_highlight_and_shade (cairo_t *cr,
-                                  const MurrineColors *colors,
+                                  const MurrineColors    *colors,
                                   const ShadowParameters *widget,
                                   int width, int height, int radius)
 {
@@ -93,6 +94,7 @@ murrine_draw_highlight_and_shade (cairo_t *cr,
 	double y = 1.0;
 	width  -= 3;
 	height -= 3;
+	radius = MIN (radius, MIN ((double)width/2.0, (double)height/2.0));
 
 	if (radius < 0)
 		radius = 0;
@@ -247,16 +249,18 @@ static void
 murrine_rgba_draw_entry (cairo_t *cr,
                          const MurrineColors    *colors,
                          const WidgetParameters *widget,
+                         const FocusParameters  *focus,
                          int x, int y, int width, int height)
 {
-	double xos = widget->xthickness > 1 ? 1 : 0;
-	double yos = widget->ythickness > 1 ? 1 : 0;
 	const MurrineRGB *base = &colors->base[widget->state_type];
 	MurrineRGB border = colors->shade[widget->disabled ? 4 : 5];
 	int radius = CLAMP (widget->roundness, 0, 3);
 
 	murrine_shade (&border, 0.92, &border);
 
+	if (widget->focus)
+		border = focus->color;
+
 	cairo_translate (cr, x+0.5, y+0.5);
 
 	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
@@ -274,8 +278,11 @@ murrine_rgba_draw_entry (cairo_t *cr,
 	/* Draw the focused border */
 	if (widget->focus)
 	{
+		MurrineRGB focus_shadow;
+		murrine_shade (&border, 1.54, &focus_shadow);
+
 		cairo_rectangle (cr, 2, 2, width-5, height-5);
-		murrine_set_color_rgba (cr, &colors->spot[1], 0.5);
+		murrine_set_color_rgba (cr, &focus_shadow, 0.5);
 		cairo_stroke(cr);
 	}
 	else if (widget->mrn_gradient.gradients)
@@ -291,7 +298,7 @@ murrine_rgba_draw_entry (cairo_t *cr,
 		cairo_stroke (cr);
 	}
 
-	murrine_set_color_rgb (cr, widget->focus ? &colors->spot[2] : &border);
+	murrine_set_color_rgb (cr, &border);
 	murrine_rounded_rectangle (cr, 1, 1, width-3, height-3, radius, widget->corners);
 	cairo_stroke (cr);
 }
@@ -1253,7 +1260,7 @@ murrine_rgba_draw_scrollbar_slider (cairo_t *cr,
 
 	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 
-	murrine_rounded_rectangle_closed (cr, 1, 1, width-2, height-2, widget->roundness, widget->corners);
+	murrine_rounded_rectangle_closed (cr, 1, 1, width-2, height-2, widget->roundness-1, widget->corners);
 	cairo_clip_preserve (cr);
 
 	murrine_draw_glaze (cr, &fill,
