@@ -223,7 +223,6 @@ murrine_style_draw_flat_box (DRAW_ARGS)
 		murrine_set_widget_parameters (widget, style, state_type, &params);
 		if (params.mrn_gradient.use_rgba)
 		{
-			params.corners = MRN_CORNER_ALL;
 			params.mrn_gradient.rgba_opacity = TOOLTIP_OPACITY;
 		}
 
@@ -363,6 +362,7 @@ murrine_style_draw_shadow (DRAW_ARGS)
 	if (DETAIL ("entry") && !(widget && widget->parent && MRN_IS_TREE_VIEW (widget->parent)))
 	{
 		WidgetParameters params;
+		FocusParameters  focus;
 
 		/* Override the entries state type, because we are too lame to handle this via
 		 * the focus ring, and GtkEntry doesn't even set the INSENSITIVE state ... */
@@ -406,7 +406,16 @@ murrine_style_draw_shadow (DRAW_ARGS)
 			}
 		}
 
-		STYLE_FUNCTION(draw_entry) (cr, &murrine_style->colors, &params,
+		/* Focus color */
+		if (murrine_style->has_focus_color)
+		{
+			ge_gdk_color_to_cairo (&murrine_style->focus_color, &focus.color);
+			focus.has_color = TRUE;
+		}
+		else
+			focus.color = colors->spot[2];
+
+		STYLE_FUNCTION(draw_entry) (cr, &murrine_style->colors, &params, &focus,
 		                            x, y, width, height);
 	}
 	else if (DETAIL ("frame") && widget && MRN_IS_STATUSBAR (widget->parent))
@@ -425,7 +434,7 @@ murrine_style_draw_shadow (DRAW_ARGS)
 			STYLE_FUNCTION(draw_statusbar) (cr, colors, &params,
 			                                x, y, width, height);
 	}
-	else if (DETAIL ("frame"))
+	else if (DETAIL ("frame") || DETAIL ("calendar"))
 	{
 		WidgetParameters params;
 		FrameParameters  frame;
@@ -465,7 +474,6 @@ murrine_style_draw_shadow (DRAW_ARGS)
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
 
-		params.corners = MRN_CORNER_ALL;
 		if (params.roundness < 2)
 			params.corners = MRN_CORNER_NONE;
 
@@ -506,7 +514,6 @@ murrine_style_draw_box_gap (DRAW_ARGS,
 
 		murrine_get_notebook_tab_position (widget, &start, &end);
 
-		params.corners = MRN_CORNER_ALL;
 		switch (gap_side)
 		{
 			case GTK_POS_TOP:
@@ -721,7 +728,8 @@ murrine_style_draw_box (DRAW_ARGS)
 		horizontal = height < 2*width;
 
 		/* This is not that great. Ideally we would have a nice vertical menubar. */
-		if (params.mrn_gradient.use_rgba || ((shadow_type != GTK_SHADOW_NONE) && horizontal))
+		if (params.mrn_gradient.use_rgba ||
+		    ((shadow_type != GTK_SHADOW_NONE) && horizontal))
 			STYLE_FUNCTION(draw_menubar) (cr, colors, &params, x, y,
 			                              width, height+offset, menubarstyle);
 	}
@@ -779,10 +787,11 @@ murrine_style_draw_box (DRAW_ARGS)
 		WidgetParameters params;
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
-		params.roundness = murrine_style->roundness;
+		params.active = shadow_type == GTK_SHADOW_IN;
 
 		boolean horizontal = TRUE;
-		if (((float)width/height<0.5)|| (murrine_style->glazestyle > 0 && width<height))
+		if (((float)width/height < 0.5) ||
+		    (murrine_style->glazestyle > 0 && width<height))
 			horizontal = FALSE;
 
 		if ((widget && (MRN_IS_COMBO_BOX_ENTRY (widget->parent) || MRN_IS_COMBO (widget->parent))))
@@ -802,8 +811,7 @@ murrine_style_draw_box (DRAW_ARGS)
 						params.corners = MRN_CORNER_BOTTOMRIGHT | MRN_CORNER_BOTTOMLEFT;
 				}
 			}
-			else
-				params.corners = MRN_CORNER_NONE;
+
 
 			if (params.xthickness > 1)
 			{
@@ -811,16 +819,10 @@ murrine_style_draw_box (DRAW_ARGS)
 					x--;
 				width++;
 			}
-		}
-		else
-			if (murrine_style->roundness > 0)
-				params.corners = MRN_CORNER_ALL;
-			else
-				params.corners = MRN_CORNER_NONE;
 
-		if (MRN_IS_TOGGLE_BUTTON (widget) &&
-		    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
-			params.active = TRUE;
+			if (murrine_style->reliefstyle > 1)
+				params.reliefstyle = 1;
+		}
 
 		STYLE_FUNCTION(draw_button) (cr, &murrine_style->colors, &params, x, y, width, height, horizontal);
 	}
@@ -867,7 +869,6 @@ murrine_style_draw_box (DRAW_ARGS)
 			state_type = GTK_WIDGET_STATE (widget);
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
-		params.roundness = murrine_style->roundness;
 
 		boolean horizontal = TRUE;
 		if (((float)width/height<0.5)|| (murrine_style->glazestyle > 0 && width<height))
@@ -888,8 +889,6 @@ murrine_style_draw_box (DRAW_ARGS)
 					params.corners = MRN_CORNER_BOTTOMRIGHT | MRN_CORNER_BOTTOMLEFT;
 			}
 		}
-		else
-			params.corners = MRN_CORNER_NONE;
 
 		if (style->xthickness > 1)
 		{
@@ -911,7 +910,7 @@ murrine_style_draw_box (DRAW_ARGS)
 		slider.horizontal = (GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL);
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
-		params.corners    = MRN_CORNER_NONE;
+		params.corners = MRN_CORNER_NONE;
 
 		STYLE_FUNCTION(draw_scale_trough) (cr, &murrine_style->colors,
 		                                   &params, &slider,
@@ -958,11 +957,6 @@ murrine_style_draw_box (DRAW_ARGS)
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
 
-		if (murrine_style->roundness > 0)
-			params.corners = MRN_CORNER_ALL;
-		else
-			params.corners = MRN_CORNER_NONE;
-
 		if (MRN_IS_RANGE (widget))
 			scrollbar.horizontal = GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL;
 
@@ -1162,11 +1156,6 @@ murrine_style_draw_box (DRAW_ARGS)
 		if (DETAIL ("vscale"))
 			horizontal = FALSE;
 
-		if (murrine_style->roundness > 0)
-			params.corners = MRN_CORNER_ALL;
-		else
-			params.corners = MRN_CORNER_NONE;
-
 		/* Use reliefstyle to remove inset on disabled slider button */
 		if (params.disabled)
 			params.reliefstyle = 0;
@@ -1193,11 +1182,6 @@ murrine_style_draw_box (DRAW_ARGS)
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
 
-		if (murrine_style->roundness > 0)
-			params.corners = MRN_CORNER_ALL;
-		else
-			params.corners = MRN_CORNER_NONE;
-
 		STYLE_FUNCTION(draw_optionmenu) (cr, colors, &params, &optionmenu, x, y, width, height);
 	}
 	else if (DETAIL ("menuitem"))
@@ -1206,11 +1190,6 @@ murrine_style_draw_box (DRAW_ARGS)
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
 
-		if (murrine_style->roundness > 0)
-			params.corners = MRN_CORNER_ALL;
-		else
-			params.corners = MRN_CORNER_NONE;
-
 		if (widget && !(MRN_IS_MENU_BAR (widget->parent) && murrine_style->menubaritemstyle))
 		{
 			if (murrine_style->menustyle != 1 || (MRN_IS_MENU_BAR (widget->parent) && !murrine_style->menubaritemstyle))
@@ -1229,11 +1208,7 @@ murrine_style_draw_box (DRAW_ARGS)
 			params.xthickness = 2;
 			params.ythickness = 2;
 			params.reliefstyle = 0;
-
-			if (murrine_style->roundness > 0)
-				params.corners = MRN_CORNER_TOPRIGHT | MRN_CORNER_TOPLEFT;
-			else
-				params.corners = MRN_CORNER_NONE;
+			params.corners = MRN_CORNER_TOPRIGHT | MRN_CORNER_TOPLEFT;
 
 			STYLE_FUNCTION(draw_button) (cr, colors, &params, x, y, width, height+1, TRUE);
 		}
@@ -1625,8 +1600,6 @@ murrine_style_draw_shadow_gap (DRAW_ARGS,
 
 		murrine_set_widget_parameters (widget, style, state_type, &params);
 
-		params.corners = MRN_CORNER_ALL;
-
 		if (params.roundness < 2)
 			params.corners = MRN_CORNER_NONE;
 
@@ -1834,7 +1807,7 @@ murrine_style_init_from_rc (GtkStyle   *style,
 			murrine_style->roundness = 0;
 	}
 	else
-		murrine_style->roundness       = MURRINE_RC_STYLE (rc_style)->roundness;
+		murrine_style->roundness = MURRINE_RC_STYLE (rc_style)->roundness;
 	murrine_style->animation           = MURRINE_RC_STYLE (rc_style)->animation;
 	murrine_style->colorize_scrollbar  = MURRINE_RC_STYLE (rc_style)->colorize_scrollbar;
 	murrine_style->has_focus_color     = MURRINE_RC_STYLE (rc_style)->has_focus_color;
@@ -2022,7 +1995,6 @@ murrine_style_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state
 	murrine_set_widget_parameters (widget, style, state_type, &params);
 
 	/* Corners */
-	params.corners = MRN_CORNER_ALL;
 	if (widget && widget->parent && MRN_IS_COMBO_BOX_ENTRY(widget->parent))
 	{
 		if (params.ltr)
@@ -2079,13 +2051,9 @@ murrine_style_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state
 			else
 				focus.type = MRN_FOCUS_BUTTON_FLAT;
 
-			/* This is a workaround for the bogus focus handling that
-			 * clearlooks has currently.
-			 * I truely dislike putting it here, but I guess it is better
-			 * then having such a visible bug. It should be removed in the
-			 * next unstable release cycle.  -- Benjamin
-			if (ge_object_is_a (G_OBJECT (widget), "ButtonWidget"))
-				focus.type = MRN_FOCUS_LABEL; */
+			/* Workaround for the panel. */
+			if (murrine_object_is_a (G_OBJECT (widget), "ButtonWidget"))
+				focus.type = MRN_FOCUS_LABEL;
 		}
 	}
 	else if (detail && g_str_has_prefix (detail, "treeview"))
